import math
import random

import pymunk
from pymunk import Vec2d

from roomrpg import cfg
from roomrpg.actors.cbodysprite import CBodySprite
from roomrpg.data import get_image


class Enemy(CBodySprite):
    def __init__(self, game, index):
        enemy_type = cfg.ENEMY_TYPES[index]

        super(Enemy, self).__init__(get_image(enemy_type['img']))

        self.cbody = pymunk.Body(1, 1)
        self.cbody.position = self.get_random_position(game)
        self.cbody.angle = self.get_random_angle()

        self.cshape = pymunk.Circle(self.cbody, enemy_type['radius'])
        self.cshape.elasticity = 1.0
        self.cshape.collision_type = cfg.COLL_ENEMY

        self.speed = enemy_type['speed']
        self.lvl = enemy_type['lvl']
        self.exp_gain = enemy_type['exp_gain']

        game.add(self)

        self.start_moving()

    def get_random_position(self, game):
        arena_padding = cfg.ARENA_PADDING
        enemy_padding = cfg.ENEMY_NOT_ALLOWED_PADDING
        return Vec2d(
            random.uniform(enemy_padding, game.geom.w - arena_padding),
            random.uniform(arena_padding, game.geom.h - arena_padding)
        )

    def get_random_angle(self):
        return random.uniform(0.0, math.pi * 2.0)

    def start_moving(self):
        impulse = Vec2d.unit()
        impulse.rotate(self.cbody.angle)
        self.cbody.apply_impulse(impulse * self.get_speed())

    def get_speed(self):
        return self.speed
