import math

import pymunk
from pymunk import Vec2d

from roomrpg import cfg
from roomrpg.actors.cbodysprite import CBodySprite
from roomrpg.data import get_image


class Player(CBodySprite):
    def __init__(self, game):
        super(Player, self).__init__(get_image('player.png'))
        self.rotation = 90.0

        self.cbody = pymunk.Body(1, 1)
        self.cbody.position = Vec2d(
            cfg.ARENA_PADDING + cfg.PLAYER_RADIUS + 10.0,
            game.geom.h / 2.0
        )
        self.cbody.angle = math.radians(-90.0)

        self.cshape = pymunk.Circle(self.cbody, cfg.PLAYER_RADIUS)
        self.cshape.elasticity = 1.0
        self.cshape.collision_type = cfg.COLL_PLAYER

        game.add(self)

        self.is_moving = False
        self.lvl = 1
        self.exp = 0
        self.step(0.0)

    def fallow_mouse(self, mouse_pos):
        self.cbody.angle = self.get_angle_with_mouse(mouse_pos)

    def move_towards(self, mouse_pos):
        angle = self.get_angle_with_mouse(mouse_pos)

        self.cbody.angle = angle

        impulse = Vec2d.unit()
        impulse.rotate(angle)

        self.cbody.apply_impulse(impulse * self.get_speed())
        self.is_moving = True

    def get_angle_with_mouse(self, mouse_pos):
        return Vec2d.unit().get_angle_between(mouse_pos - self.cbody.position)

    def get_speed(self):
        return cfg.PLAYER_SPEED

    def stop_moving(self):
        self.cbody.velocity = Vec2d.zero()
        self.is_moving = False

    def fight(self, enemy):
        if enemy.lvl > self.lvl:
            return FightStatus(False)

        exp_delta = enemy.exp_gain
        self.exp += exp_delta

        lvl_delta = self.update_level()
        return FightStatus(True, exp_delta, lvl_delta)

    def update_level(self):
        new_lvl = self.lvl
        for exp_limit, lvl in cfg.PLAYER_LEVELS:
            if self.exp >= exp_limit:
                new_lvl = lvl

        delta = new_lvl - self.lvl
        self.lvl = new_lvl
        return delta


class FightStatus(object):
    def __init__(self, is_kill, exp_delta=0, lvl_delta=0):
        self.is_kill = is_kill
        self.exp_delta = exp_delta
        self.lvl_delta = lvl_delta
