import pymunk
from pymunk.vec2d import Vec2d

from roomrpg import cfg


class Game(object):
    def __init__(self, w, h):
        self.geom = Geom(w, h)
        self.space = pymunk.Space()
        self.space.gravity = Vec2d.zero()

        self.actors = []

    def clean_actors(self):
        for actor in self.actors:
            try:
                self.space.remove(actor.cshape)
            except KeyError:
                pass  # Do not care
        self.actors = []

    def add(self, actor):
        self.space.add(actor.cshape)
        if not actor.cbody.is_static:
            self.space.add(actor.cbody)
            self.actors.append(actor)

    def step(self, dt):
        for i in range(cfg.PHYSICS_STEPS_PER_LOOP):
            self.space.step(dt / float(cfg.PHYSICS_STEPS_PER_LOOP))

        for actor in self.actors:
            actor.step(dt)


class Geom(object):
    def __init__(self, w, h):
        self.w = w
        self.h = h

    def get_size(self):
        return self.w, self.h

    def get_center(self):
        return Vec2d(self.w / 2.0, self.h / 2.0)

    def get_topleft(self):
        padd = cfg.ARENA_PADDING
        return Vec2d(padd, self.h - padd)

    def get_bottomleft(self):
        padd = cfg.ARENA_PADDING
        return Vec2d(padd, padd)

    def get_topright(self):
        padd = cfg.ARENA_PADDING
        return Vec2d(self.w - padd, self.h - padd)

    def get_bottomright(self):
        padd = cfg.ARENA_PADDING
        return Vec2d(self.w - padd, padd)

    def get_corners(self):
        return [
            self.get_bottomleft(),
            self.get_topleft(),
            self.get_topright(),
            self.get_bottomright()
        ]

    def get_corner_pairs(self):
        corners = self.get_corners()
        return zip(corners, corners[1:] + corners[:1])
