from pymunk import Vec2d

from cocos.layer import Layer
from cocos.actions import MoveBy, CallFunc, FadeOut, FadeIn, Delay
from cocos.text import Label

from roomrpg import cfg


class UI(Layer):
    def __init__(self, game, player):
        super(UI, self).__init__()

        self.player = player

        position = Vec2d(
            game.geom.w - cfg.UI_PADDING, game.geom.h - cfg.UI_PADDING
        )
        self.label_shadow = Label(
            '', position + Vec2d(2, -2),
            color=(0, 0, 0, 255),
            font_size=22,
            font_name='Munro',
            anchor_x='right',
            anchor_y='top',
        )
        self.label = Label(
            '', position,
            color=(255, 255, 255, 255),
            font_size=22,
            font_name='Munro',
            anchor_x='right',
            anchor_y='top',
        )

        self.add(self.label_shadow)
        self.add(self.label)

        self.gameover = GameOver(game)
        self.gameover.visible = False
        self.add(self.gameover)

        self.update_texts()

    def update_texts(self):
        text = cfg.UI_INFO_FMT.format(self.player.lvl, self.player.exp)
        self.label.element.text = text
        self.label_shadow.element.text = text

    def show_lost_message(self):
        self.gameover.visible = True

    def show_win_message(self):
        self.gameover.make_win()
        self.gameover.visible = True

    def show_exp_message(self, exp_delta):
        self.show_player_message(cfg.UI_EXP_GAIN_FMT.format(exp_delta))

    def show_lvl_message(self, lvl_delta):
        self.show_player_message(cfg.UI_LVL_GAIN_TXT, delay=0.2)

    def show_killed_message(self, enemy_lvl):
        self.show_player_message(
            cfg.UI_KILLED_FMT.format(enemy_lvl - self.player.lvl),
            color=(174, 12, 12, 255)
        )

    def show_player_message(self, message,
                            color=(255, 255, 255, 255),
                            delay=0.0):
        label = Label(
            message,
            self.player.cbody.position,
            color=color,
            font_size=16,
            font_name='Munro',
            anchor_x='center',
            anchor_y='center',
        )
        label.opacity = 0

        self.add(label)
        label.do(
            Delay(delay) +
            (FadeIn(0.1) | MoveBy((0, 20), 0.3)) +
            FadeOut(0.1) +
            CallFunc(self.remove_message_label, label)
        )

    def remove_message_label(self, label):
        label.kill()


class GameOver(Layer):
    def __init__(self, game):
        super(GameOver, self).__init__()

        self.main_shadow = Label(
            'Game Over', game.geom.get_center() + Vec2d(4, -4),
            color=(0, 0, 0, 255),
            font_size=56,
            font_name='Munro',
            anchor_x='center',
            anchor_y='center',
        )
        self.main = Label(
            'Game Over', game.geom.get_center(),
            color=(255, 255, 255, 255),
            font_size=56,
            font_name='Munro',
            anchor_x='center',
            anchor_y='center',
        )
        self.restart_shadow = Label(
            'Press SPACE to restart', game.geom.get_center() + Vec2d(3, -51),
            color=(0, 0, 0, 255),
            font_size=24,
            font_name='Munro',
            anchor_x='center',
            anchor_y='center',
        )
        self.restart = Label(
            'Press SPACE to restart', game.geom.get_center() + Vec2d(0, -48),
            color=(255, 255, 255, 255),
            font_size=24,
            font_name='Munro',
            anchor_x='center',
            anchor_y='center',
        )
        self.add(self.main_shadow)
        self.add(self.main)
        self.add(self.restart_shadow)
        self.add(self.restart)

    def make_win(self):
        msg = 'You Won!'
        self.main_shadow.element.text = msg
        self.main.element.text = msg
